package net.locusworks.test;

import net.locusworks.common.utils.Utils;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;
import org.junit.jupiter.params.ParameterizedTest;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.stream.Stream;

import static org.junit.jupiter.api.Assertions.*;

/**
 * Test cases for the Utils class
 * @author Isaac Parenteau
 * @since 1.0.0-RELEASE
 *
 */
public class UtilsTest {

  @Test
  public void testSafeString() {
    assertNotNull(Utils.safeString(null));
    assertTrue(Utils.safeString(null).isEmpty());
    assertFalse(Utils.safeString("hello world").isEmpty());
  }


  @Test
  public void testAreEqual() {
    String val1 = "H";
    String val2 = "H";
    assertTrue(Utils.areEqual(val1, val2));

    assertThrows(IllegalArgumentException.class, () -> Utils.areEqual("1"));
  }
  @ParameterizedTest
  @MethodSource("areEqualParams")
  public void testAreEqualWithParams(List<Object> objectList, boolean or, boolean equal) {
    assertEquals(equal, Utils.areEqual(or, objectList.toArray()));
  }

  static Stream<Arguments> areEqualParams() {
    return Stream.of(
      Arguments.of(List.of("Hello", "Hello"), false, true),
      Arguments.of(List.of("Hello", "World"), false, false),
      Arguments.of(List.of("Hello", "World", "Hello"), true, true),
      Arguments.of(List.of("Hello", "World", "Fair"), true, false)
    );
  }
  
  @Test
  public void testEmptyString() {
    assertTrue(Utils.isEmptyString(null));
    assertTrue(Utils.isEmptyString(""));
    assertTrue(Utils.isEmptyString(" "));
    assertFalse(Utils.isEmptyString("foo"));
    assertFalse(Utils.isEmptyString("   bar   "));
  }
  
  @Test
  public void testToInteger() {
    assertEquals(2, (int) Utils.toInteger("Hello word", 2));
    assertEquals(23, (int) Utils.toInteger("23", 5023));
  }

  @ParameterizedTest
  @MethodSource("validateValueParams")
  public <V> void testValidateValue(V value, boolean valid) {
    assertEquals(valid, Utils.validateValue(value));
  }

  static Stream<Arguments> validateValueParams() {
    return Stream.of(
      Arguments.of(null, false),
      Arguments.of(Collections.emptyList(), false),
      Arguments.of(Map.of(), false),
      Arguments.of(false, false),
      Arguments.of("", false),
      Arguments.of("Hello", true),
      Arguments.of(List.of("Hello"), true),
      Arguments.of(Map.of("Hello", "World"), true),
      Arguments.of(true, true)
    );
  }

  @ParameterizedTest
  @MethodSource("areValidParams")
  public void testValidateValues(List<Object> objects, boolean or, boolean valid) {
      assertEquals(valid, Utils.areValid(objects.toArray()));
      assertEquals(!valid, Utils.areNotValid(objects.toArray()));
  }

  static Stream<Arguments> areValidParams() {
    return Stream.of(
      Arguments.of(List.of("Hello", List.of("Hello"), Map.of("Hello", "World"), true), false, true),
      Arguments.of(List.of(Collections.emptyList(), Map.of(), false), false, false)
    );
  }

  @Test
  public void testBuildMap() {
    Map<String, String> mymap = Utils.buildMap(TreeMap.class, String.class, String.class, "Hello", "World");
    assertNotNull(mymap);
    assertEquals(1, mymap.size());
    assertEquals("World", mymap.get("Hello"));
  }

  @Test
  public void testIsJunitRunning() {
    assertTrue(Utils.isJUnitRunning());
  }

}
